﻿using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Settings.CommonOpinions;
using Hotline.Share.Dtos.Settings;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using MongoDB.Driver;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 常用意见接口
    /// </summary>
    public class CommonPController: BaseController
    {
        private readonly ICommonOpinionDomainService _commonOpinionDomainService;
        private readonly ISystemAreaDomainService _systemAreaDomainService;
        private readonly IMapper _mapper;

        public CommonPController(ICommonOpinionDomainService commonOpinionDomainService,ISystemAreaDomainService systemAreaDomainService, IMapper mapper)
        {
            _commonOpinionDomainService = commonOpinionDomainService;
            _systemAreaDomainService= systemAreaDomainService;
            _mapper = mapper;
        }

        /// <summary>
        /// 获取常用意见
        /// </summary>
        /// <returns></returns>
        [HttpGet("common-list")]
        public async Task<IReadOnlyList<CommonOpinion>> GetCommon(string typecode)
        {
            return await _commonOpinionDomainService.GetCommonOpinions(typecode);
        }

        /// <summary>
        /// 新增常用意见
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("add-common")]
        public async Task AddCommon([FromBody]AddCommonDto dto)
        {
            var entity = _mapper.Map<CommonOpinion>(dto);
            await _commonOpinionDomainService.AddCommonOpinion(entity,HttpContext.RequestAborted);
        }

        /// <summary>
        /// 批量删除常用意见
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("del-common")]
        public async Task DelCommon([FromBody]DelCommonDto dto)
        {
            await _commonOpinionDomainService.DelCommonOpinion(dto.Ids, HttpContext.RequestAborted);
        }

        #region 省市区
        /// <summary>
        /// 获取省市区树形
        /// </summary>
        /// <returns></returns>
        [HttpGet("tree-area")]
        [Obsolete("请调用sys/area/tree")]
        public async Task<List<SystemArea>> GetAearTree()
        {
            return await _systemAreaDomainService.GetAreaTree();
        }

        #endregion
    }
}
