﻿using Hotline.FlowEngine;
using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Workflows;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using Microsoft.AspNetCore.Mvc;

namespace Hotline.Application.FlowEngine
{
    public interface IWorkflowApplication
    {
        Task StartWorkflowAsync(StartWorkflowDto dto, string externalId, CancellationToken cancellationToken);

        /// <summary>
        /// 流转至下一节点（节点办理）
        /// </summary>
        /// <param name="dto"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task NextAsync(NextWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回至任意节点
        /// </summary>
        Task RecallAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 跳转至任意节点
        /// </summary>
        Task JumpAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 重办
        /// </summary>
        Task RedoAsync(RecallDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 查询流程开始节点的下一节点配置
        /// </summary>
        Task<DefinedStepDto> GetStartOptionsAsync(string moduleCode, CancellationToken cancellationToken);

        /// <summary>
        /// 查询节点配置可选参数
        /// </summary>
        /// <param name="stepDefine"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<NextStepOptionDto> GetNextStepOptionsAsync(StepDefine stepDefine, CancellationToken cancellationToken);
    }
}
