﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DotNetCore.CAP;
using Hotline.CallCenter.Tels;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WfModules;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;

namespace Hotline.Application.Handlers.FlowEngine
{
    public class PreviousStepHandler : INotificationHandler<PreviousNotify>
    {
        private readonly IOrderDomainService _orderDomainService;
        private readonly IOrderRepository _orderRepository;
        private readonly ICapPublisher _capPublisher;
        private readonly IMapper _mapper;

        public PreviousStepHandler(
            IOrderDomainService orderDomainService,
            IOrderRepository orderRepository,
            ICapPublisher capPublisher,
            IMapper mapper)
        {
            _orderDomainService = orderDomainService;
            _orderRepository = orderRepository;
            _capPublisher = capPublisher;
            _mapper = mapper;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(PreviousNotify notification, CancellationToken cancellationToken)
        {
            var workflow = notification.Workflow;
            var data = notification.Dto;

            switch (workflow.ModuleCode)
            {
                case WorkflowModuleConsts.OrderHandle:
                    var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                    order.CheckIfFiled();
                    _mapper.Map(workflow, order);
                    await _orderRepository.UpdateAsync(order, cancellationToken);
                    if (notification.IsOrgToCenter)
                    {
                        var dto = _mapper.Map<OrderDto>(order);
                        await _capPublisher.PublishAsync(EventNames.HotlineOrderExpiredTimeUpdate, dto, cancellationToken: cancellationToken);
                    }

                    break;
                case WorkflowModuleConsts.KnowledgeAdd:
                case WorkflowModuleConsts.KnowledgeUpdate:
                case WorkflowModuleConsts.KnowledgeDelete:
                case WorkflowModuleConsts.TelRestApply:
                    break;
            }
        }
    }
}
