﻿using DotNetCore.CAP;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WfModules;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Orders;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;

namespace Hotline.Application.Handlers.FlowEngine;

public class RecallStepHandler : INotificationHandler<RecallNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;

    public RecallStepHandler(
        IOrderDomainService orderDomainService,
        IOrderRepository orderRepository,
        ICapPublisher capPublisher,
        IMapper mapper)
    {
        _orderDomainService = orderDomainService;
        _orderRepository = orderRepository;
        _capPublisher = capPublisher;
        _mapper = mapper;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(RecallNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;
        var data = notification.Dto;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderHandle:
                var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                _mapper.Map(workflow, order);
                await _orderRepository.UpdateAsync(order, cancellationToken);

                if (notification.IsOrgToCenter)
                {
                    var dto = _mapper.Map<OrderDto>(order);
                    await _capPublisher.PublishAsync(EventNames.HotlineOrderExpiredTimeUpdate, dto, cancellationToken: cancellationToken);
                }
                
                break;
            case WorkflowModuleConsts.KnowledgeAdd:
            case WorkflowModuleConsts.KnowledgeUpdate:
            case WorkflowModuleConsts.KnowledgeDelete:
            case WorkflowModuleConsts.TelRestApply:
                break;
        }
    }
}