﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.Workflows;
using Hotline.Share.Dtos.FlowEngine;
using Mapster;

namespace Hotline.Application.Mappers;

public class WorkflowMapperConfigs : IRegister
{
    public void Register(TypeAdapterConfig config)
    {
        #region workflow

        config.ForType<UpdateDefinitionDto, Definition>()
            .Ignore(d => d.Id);

        config.ForType<WorkflowStep, WorkflowStep>()
            .Ignore(d => d.Id)
            .Ignore(d => d.PreviousId)
            .Ignore(d => d.IsMain)
            .Ignore(d => d.Status)
            .Ignore(d => d.ParentId)
            .Ignore(d => d.Handlers)
            .Ignore(d => d.Steps)
            .Ignore(d => d.StartCountersignId)
            .Ignore(d => d.CountersignId)
            .Ignore(d => d.IsStartedCountersignComplete)
            ;

        config.ForType<WorkflowStep, WorkflowTrace>()
            .Ignore(d => d.Id)
            .Ignore(d => d.ParentId)
            .Ignore(d => d.Status)
            .Map(d => d.StepId, s => s.Id);

        config.ForType<WorkflowSupplement, WorkflowSupplementDto>()
            .Map(d => d.CreationTime, x => x.CreationTime)
            .Map(d => d.Additions, x => x.Additions)
            .Map(d => d.Opinion, x => x.Opinion)
            .Map(d => d.Creator.Name, x => x.Creator.Name)
            .IgnoreNonMapped(true)
            ;

        config.ForType<Workflow, WorkflowDto>()
            //.IgnoreIf((s, d) => s.Assigns == null || !s.Assigns.Any(), d => d.AssignOrgs)
            .IgnoreIf((s, d) => s.Supplements == null || !s.Supplements.Any(), d => d.Supplements)
            .IgnoreIf((s, d) => s.Traces == null || !s.Traces.Any(), d => d.Traces)
            //.IgnoreIf((s, d) => s.StepBoxes == null || !s.StepBoxes.Any(), d => d.StepBoxes)
            //.IgnoreIf((s, d) => s.Definition == null, d => d.Definition)
            //.Map(d => d.AssignOrgs, s => string.Join(',', s.Assigns.Select(d => d.OrgName)))
            .Ignore(d => d.Definition)
            .Ignore(d => d.StepBoxes)
            //.Ignore(d=>d.Supplements)
            //.Ignore(d => d.Traces)
            ;

        config.ForType<BasicWorkflowDto, WorkflowStep>()
            .Map(d => d.NextHandlers, s => s.NextHandlers)
            .Map(d => d.NextMainHandler, s => s.NextMainHandler)
            .Map(d => d.NextStepCode, s => s.NextStepCode)
            .Map(d => d.AcceptSms, s => s.AcceptSms)
            .Map(d => d.Opinion, s => s.Opinion)
            .Map(d => d.Additions, s => s.Additions)
            .IgnoreNonMapped(true)
            ;

        config.ForType<NextWorkflowDto, Workflow>()
            .Map(d => d.RealHandlerPhone, s => s.RealHandlerPhone)
            .Map(d => d.RealHandlerName, s => s.RealHandlerName)
            .Map(d => d.RealCommunicationMode, s => s.RealCommunicationMode)
            .Map(d => d.RealCommunicationTime, s => s.RealCommunicationTime)
            .Map(d => d.RealCommunicationAddress, s => s.RealCommunicationAddress)
            .IgnoreNonMapped(true);

        #endregion
    }
}