﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Identity.Roles;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.DataPermissions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Enums.Order;
using SqlSugar;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Repository.SqlSugar.Orders
{
    public class OrderRepository : BaseRepositoryWorkflow<Order>, IOrderRepository, IScopeDependency
    {
        private readonly IDataPermissionFilterBuilder _dataPermissionFilterBuilder;

        public OrderRepository(ISugarUnitOfWork<HotlineDbContext> uow, IDataPermissionFilterBuilder dataPermissionFilterBuilder) : base(uow, dataPermissionFilterBuilder)
        {
            _dataPermissionFilterBuilder = dataPermissionFilterBuilder;
        }

        public async Task<string> AddOrderNavAsync(Order order, CancellationToken cancellationToken)
        {
            if (order.OrderType == EOrderType.MarketSupervisionBy12315)
            {
                //弥补AddNav方法没有自动指派到创建人
                order.AssignToCreator(_dataPermissionFilterBuilder.DataPermissionManager);

                if (order.AcceptType == EAcceptType.Complain)
                {
                    order.OrderComplain.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager);
                    await AddNav(order).Include(d => d.OrderComplain).ExecuteCommandAsync();
                }
                else if (order.AcceptType == EAcceptType.Report)
                {
                    order.OrderReport.InitDatePermission(_dataPermissionFilterBuilder.DataPermissionManager);
                    await AddNav(order).Include(d => d.OrderReport).ExecuteCommandAsync();
                }
                else
                {
                    throw UserFriendlyException.SameMessage("12315工单不支持该受理类型");
                }
            }
            else
            {
                await AddAsync(order, cancellationToken);
            }
            return order.Id;
        }

        public async Task UpdateOrderNavAsync(Order order, CancellationToken cancellationToken)
        {
            await UpdateAsync(order, cancellationToken);
            if (order.OrderType == EOrderType.MarketSupervisionBy12315)
            {
                if (order.AcceptType == EAcceptType.Complain)
                {
                    await Db.Insertable<OrderComplain>(order.OrderComplain).ExecuteCommandAsync();
                }
                else if (order.AcceptType == EAcceptType.Report)
                {
                    await Db.Insertable<OrderReport>(order.OrderReport).ExecuteCommandAsync();
                }
                else
                {
                    throw UserFriendlyException.SameMessage("12315工单不支持该受理类型");
                }
            }
        }

        public async Task RemoveOrderComplainAsync(OrderComplain orderComplain, CancellationToken cancellationToken)
        {
            await Db.Deleteable(orderComplain).ExecuteCommandAsync();
        }

        public async Task RemoveOrderReportAsync(OrderReport orderReport, CancellationToken cancellationToken)
        {
            await Db.Deleteable(orderReport).ExecuteCommandAsync();
        }
    }
}
