﻿using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.KnowledgeBase;

namespace Hotline.Share.Dtos.Knowledge
{
    /// <summary>
    /// 编辑
    /// </summary>
    public record UpdateKnowledgeDto : AddKnowledgeDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string Id { get; set; }
    }

    /// <summary>
    /// 新增知识
    /// </summary>
    public record AddKnowledgeDto : KnowledgeDto
    {
        /// <summary>
        /// 知识Code
        /// </summary>
        public string Code { get; set; }
    }

    /// <summary>
    /// 查询详情
    /// </summary>
    public record KnowledgeInfoDto : KnowledgeDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 知识Code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 知识分类名称
        /// </summary>
        public string KnowledgeTypeName { get; set; }

        /// <summary>
        /// 热点名称
        /// </summary>
        public string HotspotName { get; set; }

        /// <summary>
        /// 流程Id
        /// </summary>
        public string? WorkflowId { get; set; }
    }

    /// <summary>
    /// 基础
    /// </summary>
    public record KnowledgeDto
    {
        /// <summary>
        /// 知识分类
        /// </summary>
        public string KnowledgeTypeId { get; set; }

        /// <summary>
        /// 热点
        /// </summary>
        public string HotspotId { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// 摘要
        /// </summary>
        public string Summary { get; set; }

        /// <summary>
        /// 内容
        /// </summary>
        public string Content { get; set; }

        /// <summary>
        /// 是否公开
        /// </summary>
        public bool IsPublic { get; set; }

        /// <summary>
        /// 文档状态,默认草稿
        /// </summary>
        public EKnowledgeStatus Status { get; set; } = EKnowledgeStatus.Drafts;

        /// <summary>
        /// 到期时间
        /// </summary>
        public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 关键词
        /// </summary>
        public string Keywords { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; }

        /// <summary>
        /// 关联知识
        /// </summary>
        public List<string> Knowledges { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        public string HotspotExternal { get; set; }

    }

    /// <summary>
    /// 知识详情
    /// </summary>
    public record KnowledgeShowInfoDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 知识Code
        /// </summary>
        public string Code { get; set; }

        /// <summary>
        /// 流程Id
        /// </summary>
        public string? WorkflowId { get; set; }

        /// <summary>
        /// 知识分类名称
        /// </summary>
        public string KnowledgeTypeName { get; set; }

        /// <summary>
        /// 热点名称
        /// </summary>
        public string HotspotName { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// 摘要
        /// </summary>
        public string Summary { get; set; }

        /// <summary>
        /// 内容
        /// </summary>
        public string Content { get; set; }

        /// <summary>
        /// 是否公开
        /// </summary>
        public bool IsPublic { get; set; }

        /// <summary>
        /// 创建人
        /// </summary>
        public string? CreatorName { get; set; }

        /// <summary>
        /// 创建时间
        /// </summary>
        public DateTime? CreationTime { get; set; }

        /// <summary>
        /// 关键词
        /// </summary>
        public string Keywords { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; }
    }
}
