﻿using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Hotspots;
using Hotline.Share.Dtos.Org;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order
{
    public class OrderDto : UpdateOrderDto
    {
        #region 工单属性

        /// <summary>
        /// 工单状态
        /// </summary>
        public EOrderStatus Status { get; set; }
        public string StatusText => Status.GetDescription();

        /// <summary>
        /// 过期状态
        /// </summary>
        public EExpiredStatus ExpiredStatus { get; set; }

        /// <summary>
        /// 处理方式（直办、交办）
        /// </summary>
        public EProcessType ProcessType { get; set; }

        ///// <summary>
        ///// 进展情况
        ///// </summary>
        //public EProgress Progress { get; set; }

        /// <summary>
        /// 是否公开
        /// </summary>
        public bool IsPublicity { get; set; }

        /// <summary>
        /// 是否为省工单
        /// </summary>
        public bool IsProvince { get; set; }

        /// <summary>
        /// 工单编码（20220101000001）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 省工单编号（冗余）
        /// </summary>
        public string? FromProvinceNo { get; set; }

        /// <summary>
        /// 同步省工单编号（按省工单要求生成的编号，冗余）
        /// </summary>
        public string? ToProvinceNo { get; set; }

        /// <summary>
        /// 省过期时间（省工单才有）
        /// </summary>
        public DateTime? ExpiredTimeProvince { get; set; }

        /// <summary>
        /// 110工单编号（从110来的和推送给110的都用该字段记录唯一标识）
        /// </summary>
        public string? No110 { get; set; }

        /// <summary>
        /// 是否110来的工单
        /// </summary>
        public bool IsNo110 { get; set; }

        #region 流程信息

        /// <summary>
        /// 工单开始时间(受理/接办时间=流程开启时间)
        /// </summary>
        public DateTime? StartTime { get; set; }

        /// <summary>
        /// 过期时间
        /// </summary>
        public DateTime? ExpiredTime { get; set; }

        /// <summary>
        /// 交办时间(中心交部门办理时间)
        /// </summary>
        public DateTime? CenterToOrgTime { get; set; }

        /// <summary>
        /// 归档时间（暂为流程结束时间，因流程结束自动归档）
        /// </summary>
        public DateTime? FiledTime { get; set; }

        /// <summary>
        /// 办结时长（分钟）
        /// 办结时间-交办时间
        /// </summary>
        public double HandleDuration { get; set; }

        /// <summary>
        /// 办结工作日时长(分钟)
        /// </summary>
        public double HandleDurationWorkday { get; set; }

        /// <summary>
        /// 全流程时长（分钟）
        /// 归档时间-创建时间
        /// </summary>
        public double AllDuration { get; set; }

        /// <summary>
        /// 办理时间限制(如：24小时、7个工作日)
        /// </summary>
        public string? TimeLimit { get; set; }

        public int? TimeLimitCount { get; set; }

        public ETimeType? TimeLimitUnit { get; set; }

        #region 实际办理信息（节点，部门，意见）

        /// <summary>
        /// 实际办理节点code（会签状态此字段保存最外层会签发起节点code）
        /// </summary>
        public string? ActualHandleStepCode { get; set; }

        /// <summary>
        /// 实际办理节点名称（会签状态此字段保存最外层会签发起节点名称）
        /// </summary>
        public string? ActualHandleStepName { get; set; }

        /// <summary>
        /// 到达实际办理节点时间（stepBox创建时间）
        /// </summary>
        public DateTime? ActualHandleStepCreateTime { get; set; }

        /// <summary>
        /// 实际办理节点签收时间
        /// </summary>
        public DateTime? ActualHandleStepAcceptTime { get; set; }

        /// <summary>
        /// 实际办理时间
        /// </summary>
        public DateTime? ActualHandleTime { get; set; }

        /// <summary>
        /// 实际办理人id
        /// </summary>
        public string? ActualHandlerId { get; set; }

        /// <summary>
        /// 实际办理人名称
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 实际办理部门名称
        /// </summary>
        public string? ActualHandleOrgName { get; set; }

        /// <summary>
        /// 实际办理部门编码
        /// </summary>
        public string? ActualHandleOrgCode { get; set; }

        /// <summary>
        /// 实际办理部门行政区划编码
        /// </summary>
        public string? ActualHandleOrgAreaCode { get; set; }

        /// <summary>
        /// 实际办理部门行政区划名称
        /// </summary>
        public string? ActualHandleOrgAreaName { get; set; }

        /// <summary>
        /// 实际办理意见（办理中...or 最终办理意见）
        /// </summary>
        public string ActualOpinion { get; set; } = "办理中...";

        /// <summary>
        /// 真实办理人姓名（手动填写）
        /// </summary>
        public string? RealHandlerName { get; set; }

        /// <summary>
        /// 真实办理人电话（手动填写）
        /// </summary>
        public string? RealHandlerPhone { get; set; }

        /// <summary>
        /// 沟通方式（手动填写）
        /// </summary>
        public ERealCommunicationMode? RealCommunicationMode { get; set; }

        /// <summary>
        /// 沟通时间（手动填写）
        /// </summary>
        public DateTime? RealCommunicationTime { get; set; }

        /// <summary>
        /// 沟通地点（手动填写）
        /// </summary>
        public string? RealCommunicationAddress { get; set; }

        #endregion

        #region 当前办理节点信息

        /// <summary>
        /// 当前办理节点code（非会签：当前被指派节点，会签：会签发起节点）
        /// </summary>
        public string? CurrentStepCode { get; set; }

        /// <summary>
        /// 当前节点名称
        /// </summary>
        public string? CurrentStepName { get; set; }

        /// <summary>
        /// 到达当前节点时间（stepBox创建时间）
        /// </summary>
        public DateTime? CurrentStepCreateTime { get; set; }

        #endregion

        #region 一级部门

        /// <summary>
        /// 一级部门code
        /// </summary>
        public string? OrgLevelOneCode { get; set; }

        /// <summary>
        /// 一级部门名称
        /// </summary>
        public string? OrgLevelOneName { get; set; }

        #endregion

        #region 受理人(开启流程的话务员)

        /// <summary>
        /// 受理人id
        /// </summary>
        public string? AcceptorId { get; set; }

        /// <summary>
        /// 受理人名称
        /// </summary>
        public string? AcceptorName { get; set; }

        /// <summary>
        /// 受理人工号
        /// </summary>
        public string? AcceptorStaffNo { get; set; }

        /// <summary>
        /// 受理人部门编码
        /// </summary>
        public string? AcceptorOrgCode { get; set; }

        /// <summary>
        /// 受理人部门名称
        /// </summary>
        public string? AcceptorOrgName { get; set; }

        #endregion

        #endregion


        #endregion

        public string CreationTime { get; set; }


        public string? ContactMask { get; set; }

        public string SmsSendedText => SmsSended ? "已发送" : "未发送";

        public string WorkflowId { get; set; }
        public WorkflowDto Workflow { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public string FromGenderText => FromGender.GetDescription();

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public string IdentityTypeText => IdentityType.GetDescription();

        /// <summary>
        /// 工单类型
        /// </summary>
        public string OrderTypeText => OrderType.GetDescription();

        /// <summary>
        /// 受理类型
        /// </summary>
        public string AcceptTypeText => AcceptType.GetDescription();

        /// <summary>
        /// 紧急程度
        /// </summary>
        public string EmergencyLevelText => EmergencyLevel.GetDescription();

        /// <summary>
        /// 超期时间描述（需求：超期3天、0.5天后超期）
        /// </summary>
        public string ExpiredText => CalculateExpiredText();

        private string CalculateExpiredText()
        {
            //todo 完整处理方案：1.创建待过期表，Id,过期时间，即将过期时间等字段。2.延迟消息通知处理过期，删除子表数据，处理order过期状态字段。3.此处即可采用expiredStatus进行判断
            //todo 目前暂时采用过期时间计算
            if (!ExpiredTime.HasValue)
                return string.Empty;
            var days = (DateTime.Now - ExpiredTime.Value).TotalDays;
            switch (days)
            {
                case > 0:
                    return $"超期{days:N1}天";
                case < 0:
                    return $"{days:N1}天后超期";
                default:
                    return "超期不到1天";
            }
        }

        public HotspotDto? Hotspot { get; set; }

    }

    public class UpdateOrderDto : AddOrderDto
    {
        public string Id { get; set; }
    }

    public class AddOrderDto : Position
    {
        #region 来电信息

        /// <summary>
        /// 来源渠道
        /// </summary>
        public string SourceChannel { get; set; }
        public string SourceChannelCode { get; set; }

        /// <summary>
        /// 渠道为电话时，此字段存在
        /// </summary>
        public string? CallId { get; set; }

        /// <summary>
        /// 来电归属地
        /// </summary>
        public string? CallAddress { get; set; }

        /// <summary>
        /// 来电号码
        /// </summary>
        public string? FromPhone { get; set; }

        /// <summary>
        /// 转接号码（转接来源）
        /// </summary>
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 来电/信人姓名
        /// </summary>
        public string FromName { get; set; }

        /// <summary>
        /// 来电/信人性别
        /// </summary>
        public EGender FromGender { get; set; }

        /// <summary>
        /// 来电/信人身份
        /// </summary>
        public EIdentityType IdentityType { get; set; }

        /// <summary>
        /// 证件类型
        /// </summary>
        public string? LicenceTypeCode { get; set; }

        public string? LicenceType { get; set; }

        /// <summary>
        /// 证件号码
        /// </summary>
        public string? LicenceNo { get; set; }

        /// <summary>
        /// 年龄段
        /// </summary>
        public string? AgeRangeCode { get; set; }

        public string? AgeRange { get; set; }

        /// <summary>
        /// 联系电话
        /// </summary>
        public string? Contact { get; set; }

        /// <summary>
        /// 是否接受短信，勾选校验手机号
        /// </summary>
        public bool AcceptSms { get; set; }

        /// <summary>
        /// 是否已发送短信
        /// </summary>
        public bool SmsSended { get; set; }

        /// <summary>
        /// 是否保密
        /// </summary>
        public bool IsSecret { get; set; }

        /// <summary>
        /// 工作单位(当“来电/信人身份”为“企业”时必填，其他情况非必填)
        /// </summary>
        public string? Company { get; set; }

        #endregion

        #region 诉求信息

        /// <summary>
        /// 工单类型
        /// </summary>
        public EOrderType OrderType { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public EAcceptType AcceptType { get; set; }

        /// <summary>
        /// 紧急程度
        /// </summary>
        public EEmergencyLevel EmergencyLevel { get; set; } = EEmergencyLevel.Normal;

        public string Title { get; set; }

        #region 热点

        /// <summary>
        /// 热点
        /// </summary>
        public string HotspotId { get; set; }
        public string HotspotName { get; set; }
        public string HotspotSpliceName { get; set; }

        /// <summary>
        /// 外部数据（为前端提供级联功能）
        /// </summary>
        public string HotspotExternal { get; set; }

        #endregion

        /// <summary>
        /// 事发时间
        /// </summary>
        public DateTime? IncidentTime { get; set; }

        /// <summary>
        /// 重复工单Id
        /// </summary>
        public List<string>? DuplicateIds { get; set; }

        /// <summary>
        /// 推送分类
        /// </summary>
        public string? PushTypeCode { get; set; }

        public string? PushType { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string> Additions { get; set; } = new();

        /// <summary>
        /// 诉求内容
        /// </summary>
        public string Content { get; set; }

        #endregion

        /// <summary>
        /// 来源，区分省平台或110等其他平台同步过来的工单
        /// </summary>
        public ESource Source { get; set; }

        /// <summary>
        /// 投诉
        /// </summary>
        public OrderComplainDto OrderComplain { get; set; }

        /// <summary>
        /// 举报
        /// </summary>
        public OrderReportDto OrderReport { get; set; }

        /// <summary>
        /// 附件列表
        /// </summary>
        public List<FileDto> File { get; set; } = new();
    }


    public class PublishDto : OrderDto
    {
        /// <summary>
        /// 发布范围
        /// </summary>
        public EPublishState? PublishState { get; set; }

        /// <summary>
        /// 整理内容
        /// </summary>
        public string? ArrangeContent { get; set; }
    }


    public class PublishOrderPageBaseDto
    {
        /// <summary>
        /// 来源 0：热线平台  1：省平台 2：110
        /// </summary>
        public string SourceChannel { get; set; }

        /// <summary>
        /// 工单标题
        /// </summary>
        public string OrderTitle { get; set; }

        /// <summary>
        /// 工单内容
        /// </summary>
        public string Content { get; set; }

        /// <summary>
        /// 办理结果
        /// </summary>
        public string ActualOpinion { get; set; }

        /// <summary>
        /// 实际办理部门名称
        /// </summary>
        public IdName ActualHandleOrgName { get; set; }

        /// <summary>
        /// 需回访部门
        /// </summary>
        public List<IdName>? idNames { get; set; }
    }


    public class PublishOrderDto
    {
        /// <summary>
        /// 工单ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 发布范围
        /// </summary>
        public bool PublishState { get; set; }

        /// <summary>
        /// 整理标题
        /// </summary>
        public string ArrangeTitle { get; set; }

        /// <summary>
        /// 整理内容
        /// </summary>
        public string ArrangeContent { get; set; }

        /// <summary>
        /// 整理结果
        /// </summary>
        public string ArrangeOpinion { get; set; }

        /// <summary>
        /// 需回访部门
        /// </summary>
        public List<IdName> IdNames { get; set; }

        #region 省工单使用

        /// <summary>
        /// 省是否公开
        /// </summary>
        public bool? ProPublishState { get; set; }
        /// <summary>
        /// 反馈人电话
        /// </summary>
        public string? FeedBackPhone { get; set; }

        /// <summary>
        /// 不公开原因
        /// </summary>
        public string? NoPubReason { get; set; }

        #endregion
    }


    public class PublishVisitDto
    {
        /// <summary>
        /// 工单对象
        /// </summary>
        public OrderDto Order { get; set; }

        /// <summary>
        /// 服务工单编号
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 回访评价方式
        /// </summary>
        public EVisitType? VisitType { get; set; }

        /// <summary>
        /// 回访人
        /// </summary>
        public string VisitName { get; set; }

        /// <summary>
        /// 回访时间
        /// </summary>
        public DateTime? VisitTime { get; set; }

        /// <summary>
        /// 回访评价情况
        /// </summary>
        public string VisitRemark { get; set; }

        /// <summary>
        /// 回访结果满意度
        /// </summary>
        public string SubjectResultSatifyCode { get; set; }

        /// <summary>
        /// 行政区划代码
        /// </summary>
        public string AreaCode { get; set; }

        /// <summary>
        /// 工单回访首次结果满意度
        /// </summary>
        public string FirstSatisfactionCode { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public string ClientGuid { get; set; }
	}

    public class SuperviseOrderDto {

        /// <summary>
        /// 督办ID
        /// </summary>
		public string Id { get; set; }

        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        /// <summary>
        /// 督办回复时限
        /// </summary>
        public DateTime ReplyLimitTime { get; set; }

        /// <summary>
        /// 被督办部门ID
        /// </summary>
        public string OrgId { get; set; }

        /// <summary>
        /// 被督办部门名称
        /// </summary>
        public string OrgName { get; set; }


        /// <summary>
        /// 创建部门名称
        /// </summary>
        public string? CreatorOrgName { get; set; }

		/// <summary>
		/// 督办状态
		/// </summary>
		public int State { get; set; }

        /// <summary>
        /// 督办签收时间
        /// </summary>
        public DateTime? SignTime { get; set; }

        /// <summary>
        /// 督办回复时间
        /// </summary>
        public DateTime? ReplyTime { get; set; }

        /// <summary>
        /// 督办申请内容
        /// </summary>
        public string? ApplyContent { get; set; }

        /// <summary>
        /// 督办回复内容
        /// </summary>
        public string? ReplyContent { get; set; }

        /// <summary>
        /// 回复人
        /// </summary>
        public string? ReplyId { get; set; }

        /// <summary>
        /// 附件
        /// </summary>
        public List<string>? Additions { get; set; } = new();

        public DateTime? LastModificationTime { get; set; }

        public string? CreatorId { get; set; }

        public string? CreatorOrgId { get; set; }

        public DateTime CreationTime { get; set; }

		/// <summary>
		/// 工单对象
		/// </summary>
		public OrderDto Order { get; set; }
	}


    public class UrgeOrderDto
    {

	    /// <summary>
	    /// 催办ID
	    /// </summary>
	    public string Id { get; set; }

	    /// <summary>
	    /// 工单ID
	    /// </summary>
	    public string OrderId { get; set; }

	    /// <summary>
	    /// 催办回复时限
	    /// </summary>
	    public DateTime ReplyLimitTime { get; set; }

	    /// <summary>
	    /// 被催办部门ID
	    /// </summary>
	    public string OrgId { get; set; }

		/// <summary>
		/// 被催办部门名称
		/// </summary>
		public string OrgName { get; set; }

		/// <summary>
		/// 创建部门名称
		/// </summary>
		public string? CreatorOrgName { get; set; }

		/// <summary>
		/// 催办状态
		/// </summary>
		public int State { get; set; }

	    /// <summary>
	    /// 催办签收时间
	    /// </summary>
	    public DateTime? SignTime { get; set; }

	    /// <summary>
	    /// 催办回复时间
	    /// </summary>
	    public DateTime? ReplyTime { get; set; }

	    /// <summary>
	    /// 催办申请内容
	    /// </summary>
	    public string? ApplyContent { get; set; }

	    /// <summary>
	    /// 催办回复内容
	    /// </summary>
	    public string? ReplyContent { get; set; }

	    /// <summary>
	    /// 回复人
	    /// </summary>
	    public string? ReplyId { get; set; }

	    /// <summary>
	    /// 附件
	    /// </summary>
	    public List<string>? Additions { get; set; } = new();

	    public DateTime? LastModificationTime { get; set; }

	    public string? CreatorId { get; set; }

	    public string? CreatorOrgId { get; set; }

	    public DateTime CreationTime { get; set; }

	    /// <summary>
	    /// 工单对象
	    /// </summary>
	    public OrderDto Order { get; set; }
	}
}
