﻿using Hotline.Share.Dtos;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;

namespace Hotline.FlowEngine.Definitions;

/// <summary>
/// 节点基础信息
/// </summary>
public class StepBasic
{
    public string Name { get; set; }

    /// <summary>
    /// 模板内唯一
    /// </summary>
    public string Code { get; set; }

    /// <summary>
    /// 办理/汇总、开始/结束
    /// </summary>
    public EStepType StepType { get; init; }

    /// <summary>
    /// 业务属性
    /// </summary>
    public EBusinessProperty BusinessProperty { get; set; }

    /// <summary>
    /// 办理者类型
    /// </summary>
    public EHandlerType HandlerType { get; set; }

    /// <summary>
    /// 办理者分类（或是直接保存办理者）
    /// <example>
    /// 根据类型可能为：roles, orgLevels, orgTypes, orgCodes, userIds
    /// </example>
    /// </summary>
    public List<IdName> HandlerClassifies { get; set; } = new();

    /// <summary>
    /// 当前环节办理会签模式
    /// </summary>
    public ECountersignMode CountersignMode { get; set; }

    /// <summary>
    /// 发起会签节点code
    /// </summary>
    public string? CountersignStartStepCode { get; set; }

    /// <summary>
    /// 会签汇总节点code
    /// </summary>
    public string? CountersignEndStepCode { get; set; }

    /// <summary>
    /// 节点组件配置
    /// </summary>
    public List<string> Components { get; set; } = new();

    /// <summary>
    /// 自动路径策略
    /// </summary>
    [SugarColumn(DefaultValue = "0")]
    public EPathPolicy PathPolicy { get; set; }
}