﻿using Hotline.FlowEngine.Definitions;
using Hotline.FlowEngine.WfModules;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Enums.FlowEngine;
using XF.Domain.Entities;

namespace Hotline.FlowEngine.Workflows
{
    public interface IWorkflowDomainService
    {
        /// <summary>
        /// 创建流程
        /// </summary>
        Task<Workflow> CreateWorkflowAsync(WorkflowModule wfModule, string title, string userId, string userCode, string? externalId = null, CancellationToken cancellationToken = default);

        /// <summary>
        /// 进行流程的开始节点
        /// </summary>
        Task StartAsync(Workflow workflow, BasicWorkflowDto dto, StepDefine nextStepBoxDefine, FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

        /// <summary>
        /// 查询工作流
        /// </summary>
        Task<Workflow> GetWorkflowAsync(string workflowId, bool withDefine = false, bool withSteps = false, bool withTraces = false,
            bool withSupplements = false, bool withAssigns = false, bool withCountersigns = false, CancellationToken cancellationToken = default);

        /// <summary>
        /// 查询工作流包含当前用户办理权限（是否可办理）
        /// </summary>
        Task<(Workflow, bool)> GetWorkflowHandlePermissionAsync(string workflowId, string userId, string orgCode, bool withDefine = false, bool withSteps = false, bool withTraces = false,
            bool withSupplements = false, bool withAssigns = false, bool withCountersigns = false, CancellationToken cancellationToken = default);

        /// <summary>
        /// 受理,接办
        /// </summary>
        Task AcceptAsync(Workflow workflow, string userId, string userName, string orgCode, string orgName, CancellationToken cancellationToken);

        /// <summary>
        /// 办理（流转至下一节点）
        /// </summary>
        Task NextAsync(Workflow workflow, NextWorkflowDto dto, StepDefine nextStepBoxDefine,
            bool isStartCountersign, FlowAssignInfo flowAssignInfo, DateTime expiredTime, CancellationToken cancellationToken);

        /// <summary>
        /// 退回（返回前一节点）
        /// </summary>
        /// <returns></returns>
        Task PreviousAsync(Workflow workflow, PreviousWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 撤回（返回到之前任意节点）
        /// </summary>
        Task RecallAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

        /// <summary>
        /// 跳转（直接将流程跳转至任意节点）
        /// </summary>
        Task JumpAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, bool isStartCountersign, FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

        /// <summary>
        /// 重办
        /// </summary>
        Task RedoAsync(Workflow workflow, RecallDto dto, StepDefine targetStepDefine, FlowAssignInfo flowAssignInfo, CancellationToken cancellationToken);

        /// <summary>
        /// 补充
        /// </summary>
        /// <returns></returns>
        Task SupplementAsync(Workflow workflow, EndWorkflowDto dto, CancellationToken cancellationToken);

        /// <summary>
        /// 终止流程
        /// </summary>
        Task TerminateAsync(TerminateDto dto, CancellationToken cancellationToken);

        StepDefine GetStepBoxDefine(Definition definition, string stepCode);

        /// <summary>
        /// 查询当前待办节点的下一级节点配置(办理参数)
        /// </summary>
        IReadOnlyList<StepDefine> GetNextStepDefines(Workflow workflow);

        /// <summary>
        /// 查找当前办理节点
        /// </summary>
        (WorkflowStep StepBox, WorkflowStep Step) FindCurrentStep(Workflow workflow);

        /// <summary>
        /// 查询待回访部门
        /// </summary>
        Task<(IdName, IReadOnlyList<IdName>)> GetUnvisitOrgsAsync(string workflowId,
            CancellationToken cancellationToken);

        /// <summary>
        /// 更新一级部门信息
        /// </summary>
        Task UpdateOrgLevelOneAsync(Workflow workflow, string orgCode, string orgName, CancellationToken cancellationToken);

        /// <summary>
        /// 依据配置过滤下一节点
        /// </summary>
        List<StepDefine> NextStepDefineFilter(EPathPolicy pathPolicy, List<StepDefine> nextStepDefines);

        /// <summary>
        /// 撤销流程
        /// </summary>
        Task CancelAsync(CancelDto dto, CancellationToken cancellationToken);
    }
}
