﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Share.Enums.FlowEngine;
using SqlSugar;
using XF.Domain.Repository;

namespace Hotline.FlowEngine.Workflows
{
    public class WorkflowCountersign : CreationEntity
    {
        public string WorkflowId { get; set; }

        /// <summary>
        /// 发起会签节点id（从开始节点直接发起的会签，该id和code为start节点的id与code）
        /// </summary>
        public string StartStepId { get; set; }

        //冗余
        public string StartStepCode { get; set; }

        /// <summary>
        /// 业务属性
        /// </summary>
        [SugarColumn(DefaultValue = "0")]
        public EBusinessProperty StartStepBusiProperty { get; set; }

        /// <summary>
        /// 会签汇总节点id
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? EndStepId { get; set; }

        //冗余
        [SugarColumn(IsNullable = true)]
        public string? EndStepCode { get; set; }

        /// <summary>
        /// 业务属性
        /// </summary>
        [SugarColumn(DefaultValue = "0")]
        public EBusinessProperty EndStepBusiProperty { get; set; }

        /// <summary>
        /// 会签结束时间
        /// </summary>
        public DateTime? CompleteTime { get; set; }

        /// <summary>
        /// 会签嵌套会签场景记录上级会签Id
        /// </summary>
        [SugarColumn(IsNullable = true)]
        public string? ParentId { get; set; }

        /// <summary>
        /// 该会签参与成员数量
        /// </summary>
        public int Members { get; set; }

        /// <summary>
        /// 会签类型
        /// </summary>
        [SugarColumn(DefaultValue = "0")]
        public ECounterSignType CounterSignType { get; set; }

        #region method

        /// <summary>
        /// 会签是否完成(如有嵌套会签，表示下级所有会签都完成了)
        /// </summary>
        public bool IsCompleted() => CompleteTime.HasValue;

        /// <summary>
        /// 结束会签（需提前确认所有子集都已结束）
        /// </summary>
        /// <param name="endStepId"></param>
        /// <param name="endStepCode"></param>
        public void Complete(string endStepId, string endStepCode, EBusinessProperty businessProperty)
        {
            EndStepId = endStepId;
            EndStepCode = endStepCode;
            EndStepBusiProperty = businessProperty;
            CompleteTime = DateTime.Now;
        }

        #endregion
    }
}
