﻿using Hotline.Settings;
using Hotline.Settings.Hotspots;
using Hotline.Share.Enums.KnowledgeBase;
using Hotline.Users;
using SqlSugar;
using System.ComponentModel;
using XF.Domain.Repository;

namespace Hotline.KnowledgeBase;

/// <summary>
/// 知识库文档内容
/// </summary>
[Description("知识库文档")]
public class Knowledge : WorkflowEntity//   WorkflowEntity  FullStateEntity
{
    /// <summary>
    /// 知识分类
    /// </summary>
    public string KnowledgeTypeId { get; set; }

    /// <summary>
    /// 热点
    /// </summary>
    public string HotspotId { get; set; }

    /// <summary>
    /// 知识编号
    /// </summary>
    public string Code { get; set; }

    /// <summary>
    /// 标题
    /// </summary>
    public string Title { get; set; }

    /// <summary>
    /// 摘要
    /// </summary>
    [SugarColumn(ColumnDataType = "varchar(1000)", IsNullable = true)]
    public string Summary { get; set; }

    /// <summary>
    /// 内容
    /// </summary>
    [SugarColumn(ColumnDataType = "text")]
    public string Content { get; set; }

    /// <summary>
    /// 浏览量
    /// </summary>
    public int PageView { get; set; } = 0;

    /// <summary>
    /// 是否公开
    /// </summary>
    public bool IsPublic { get; set; }

    /// <summary>
    /// 文档状态
    /// </summary>
    public EKnowledgeStatus Status { get; set; }

    /// <summary>
    /// 上架时间
    /// </summary>
    public DateTime? OnShelfTime { get; set; }

    /// <summary>
    /// 下架时间
    /// </summary>
    public DateTime? OffShelfTime { get; set; }

    /// <summary>
    /// 过期时间
    /// </summary>
    public DateTime? ExpiredTime { get; set; }

    /// <summary>
    /// 关键词
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string Keywords { get; set; }

    /// <summary>
    /// 版本号
    /// </summary>
    public int Version { get; set; } = 0;

    /// <summary>
    /// 外部数据（为前端提供级联功能）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? HotspotExternal { get; set; }

    /// <summary>
    /// 附件
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true, IsNullable = true)]
    public List<string> Additions { get; set; }

    /// <summary>
    /// 关联知识
    /// </summary>
    [SugarColumn(ColumnDataType = "json", IsJson = true,IsNullable =true)]
    public List<string> Knowledges { get; set; }

    /// <summary>
    /// 分类
    /// </summary>
    [Navigate(NavigateType.OneToOne, nameof(KnowledgeTypeId))]//一对一 
    public KnowledgeType KnowledgeType { get; set; }

    /// <summary>
    /// 部门
    /// </summary>
    [Navigate(NavigateType.OneToOne, nameof(CreatorOrgId))]//一对一 
    public SystemOrganize SystemOrganize { get; set; }

    /// <summary>
    /// 用户
    /// </summary>
    [Navigate(NavigateType.OneToOne, nameof(CreatorId))]//一对一 
    public User User { get; set; }

    /// <summary>
    /// 热点
    /// </summary>
    [Navigate(NavigateType.OneToOne, nameof(HotspotId))]//一对一 
    public Hotspot HotspotType { get; set; }
}