﻿using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Settings.TimeLimits
{
    public class TimeLimitDomainService : ITimeLimitDomainService, IScopeDependency
    {
        private readonly ITimeLimitRepository _timeLimitRepository;
        private readonly IDaySettingRepository _daySettingRepository;
        private List<CalcTimeSetting> timeSettingList;

        public TimeLimitDomainService(ITimeLimitRepository timeLimitRepository,IDaySettingRepository daySettingRepository)
        {
            _timeLimitRepository = timeLimitRepository;
            _daySettingRepository = daySettingRepository;

            //初始化常量
            timeSettingList = new List<CalcTimeSetting>();
            CalcTimeSetting setting1 = new CalcTimeSetting() { BugCode = "", BusName = "咨询", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting1);
            CalcTimeSetting setting2 = new CalcTimeSetting() { BugCode = "", BusName = "举报", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting2);
            CalcTimeSetting setting3 = new CalcTimeSetting() { BugCode = "", BusName = "投诉", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting3);
            CalcTimeSetting setting4 = new CalcTimeSetting() { BugCode = "", BusName = "求助", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting4);
            CalcTimeSetting setting5 = new CalcTimeSetting() { BugCode = "", BusName = "建议", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting5);
            CalcTimeSetting setting6 = new CalcTimeSetting() { BugCode = "", BusName = "意见", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting6);
            CalcTimeSetting setting7 = new CalcTimeSetting() { BugCode = "", BusName = "惠民帮助", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting7);
            CalcTimeSetting setting8 = new CalcTimeSetting() { BugCode = "", BusName = "大气污染举报", TimeType = ETimeType.WorkDay, TimeValue = 5 };
            timeSettingList.Add(setting8);
            CalcTimeSetting setting9 = new CalcTimeSetting() { BugCode = "", BusName = "表扬", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting9);
            CalcTimeSetting setting10 = new CalcTimeSetting() { BugCode = "", BusName = "其他", TimeType = ETimeType.WorkDay, TimeValue = 3 };
            timeSettingList.Add(setting10);
        }

        public async Task<string> AddAsync(TimeLimit model, CancellationToken cancellationToken)
        {
            return await _timeLimitRepository.AddAsync(model, cancellationToken);
        }

        public async Task UpdateAsync(TimeLimit model, CancellationToken cancellationToken)
        {
            var dbModel = await _timeLimitRepository.GetAsync(model.Id, cancellationToken);
            if (dbModel == null)
            {
                throw new UserFriendlyException("无效数据");
            }
            await _timeLimitRepository.UpdateAsync(model, cancellationToken);
        }

        /// <summary>
        /// 计算工作时间分钟数
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="endTime"></param>
        /// <returns></returns>
        public int CalcWorkTime(DateTime beginTime, DateTime endTime)
        {
            //计算时间差
            TimeSpan minuteSpan = new TimeSpan(endTime.Ticks - beginTime.Ticks);
            //总时差分钟数
            int minutes = (int)minuteSpan.TotalMinutes;


            //计算开始时间是否计算在工作时间内
            if (!IsWorkDay(beginTime))
            {
                var newBeginTime = beginTime.AddDays(1).Date;
                TimeSpan beginTimeMinuteSpan = new TimeSpan(newBeginTime.Ticks - beginTime.Ticks);
                int beginTimeMinute = (int)beginTimeMinuteSpan.TotalMinutes;
                minutes = minutes - beginTimeMinute;
            }


            //计算结束时间是否计算在工作时间内
            if (!IsWorkDay(endTime))
            {
                var newEndTime = endTime.Date;
                TimeSpan endTimeMinuteSpan = new TimeSpan(endTime.Ticks - newEndTime.Ticks);
                int endTimeMinute = (int)endTimeMinuteSpan.TotalMinutes;
                minutes = minutes - endTimeMinute;
            }

            //计算整天时间是否在工作时间内
            int days = minuteSpan.Days;
            for (int i = 0; i < days; i++)
            {
                var newDay = beginTime.AddDays(i + 1);
                if (newDay.Date == endTime.Date)
                {
                    continue;
                }

                if (!IsWorkDay(beginTime.AddDays(i + 1)))
                {
                    minutes = minutes - 1440;
                }
            }
            return minutes;
        }


        /// <summary>
        /// 计算工作日
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="busCode"></param>
        /// <returns></returns>
        public TimeResult? CalcEndTime(DateTime beginTime, string busCode)
        {
            var setting = timeSettingList.FirstOrDefault(x => x.BugCode == busCode);
            if (setting != null)
            {
                return CalcEndTime(beginTime, setting.TimeType, setting.TimeValue);
            }
            return null;
        }


        /// <summary>
        /// 计算工作日
        /// </summary>
        /// <param name="beginTime"></param>
        /// <param name="timeType"></param>
        /// <param name="timeValue"></param>
        /// <returns></returns>
        public TimeResult CalcEndTime(DateTime beginTime, ETimeType timeType, int timeValue)
        {
            switch (timeType)
            {
                //新增对应小时
                case ETimeType.Hour:
                    return new TimeResult { EndTime = beginTime.AddHours(timeValue), RuleStr = timeValue + "小时" };
                //新增工作日
                case ETimeType.WorkDay:
                    //检查时间段内是否存在休息日或者工作日
                    int day = 1;
                    for (int i = 1; i < timeValue + 1; i++)
                    {
                        if (IsWorkDay(beginTime.AddDays(day)))
                        {
                            beginTime = beginTime.AddDays(day);
                            day = 1;
                        }
                        else
                        {
                            i--;
                            day++;
                        }
                    }
                    return new TimeResult { EndTime = beginTime, RuleStr = timeValue + "个工作日" };
                //新增自然日
                case ETimeType.Day:
                    return new TimeResult { EndTime = beginTime.AddDays(timeValue), RuleStr = timeValue + "个自然日" };
                default:
                    return null;
            }
        }

        private bool IsWorkDay(DateTime date)
        {
            return _daySettingRepository.IsWorkDay(date).GetAwaiter().GetResult();
        }


    }
}
