﻿using EasyCaching.Core;
using XF.Domain.Dependency;
using XF.Domain.Queues;

namespace XF.EasyCaching;

public class DistributedQueue : IQueue, IScopeDependency
{
    private readonly IRedisCachingProvider _redisCaching;

    public DistributedQueue(IRedisCachingProvider redisCaching)
    {
        _redisCaching = redisCaching;
    }

    public void Enqueue<TItem>(string queueName, TItem item)
    {
        _redisCaching.LPush(queueName, new List<TItem> { item });
    }

    public void Enqueue<TItem>(string queueName, IList<TItem> items)
    {
        _redisCaching.LPush(queueName, items);
    }

    public Task EnqueueAsync<TItem>(string queueName, TItem item, CancellationToken cancellationToken = default)
    {
        return _redisCaching.LPushAsync(queueName, new List<TItem> { item });
    }

    public Task EnqueueAsync<TItem>(string queueName, IList<TItem> items, CancellationToken cancellationToken = default)
    {
        return _redisCaching.LPushAsync(queueName, items);
    }

    public TItem Dequeue<TItem>(string queueName)
    {
        return _redisCaching.RPop<TItem>(queueName);
    }

    public Task<TItem> DequeueAsync<TItem>(string queueName, CancellationToken cancellationToken = default)
    {
        return _redisCaching.RPopAsync<TItem>(queueName);
    }

    public long Count(string queueName)
    {
        return _redisCaching.LLen(queueName);
    }

    public Task<long> CountAsnyc(string queueName, CancellationToken cancellationToken = default)
    {
        return _redisCaching.LLenAsync(queueName);
    }

    public void Remove<TItem>(string queueName, TItem item, bool fromHead = false, long count = 0)
    {
        _redisCaching.LRem(queueName, fromHead ? count : -count, item);
    }

    public Task RemoveAsync<TItem>(string queueName, TItem item, bool fromHead = false, long count = 0,
        CancellationToken cancellationToken = default)
    {
        return _redisCaching.LRemAsync(queueName, fromHead ? count : -count, item);
    }
}